#ifndef PKG_PACKAGE_H
#define PKG_PACKAGE_H

/** \file
 * Provide a very lightweight Package class that represent a package, with
 * optional version, with very cheap value copy operations.
 */

/*
 * Copyright (C) 2007  Enrico Zini <enrico@enricozini.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <string>

namespace pkg
{

class Implementation;

/**
 * Lightweight Package class that represent a package, with optional version,
 * with very cheap value copy operations.
 *
 * This class can be used to query package information from various information
 * sources.  The purpose is create a middle ground that makes sure that all
 * sort of different information sources about packages are referring to the
 * same package.
 */
class Package
{
protected:
	Implementation* impl;

public:
	/**
	 * Create an invalid package
	 */
	Package();

	/**
	 * Create a versionless package
	 */
	Package(const std::string& name);

	/**
	 * Create a package with a version
	 */
	Package(const std::string& name, const std::string& version);

	~Package();

	/**
	 * Return the package name
	 */
	std::string name() const;

	/**
	 * Return the package version, or the empty string if this is a
	 * versionless package.
	 */
	std::string version() const;

	/**
	 * Return true if this package contains a valid value
	 */
	bool isValid() const;

	/**
	 * Return true if this package has version information
	 */
	bool hasVersion() const;

	/**
	 * Comparison operators
	 */
	bool operator==(const Package& pkg) const;
	bool operator!=(const Package& pkg) const;
	bool operator<=(const Package& pkg) const;
	bool operator<(const Package& pkg) const;
	bool operator>=(const Package& pkg) const;
	bool operator>(const Package& pkg) const;
};

};

// vim:set ts=4 sw=4:
#endif
