#ifndef PKG_PACKAGE_STORE_H
#define PKG_PACKAGE_STORE_H

/** \file
 * Package storage implementation to support the Package flyweight objects.
 */

/*
 * Copyright (C) 2007  Enrico Zini <enrico@enricozini.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <string>
#include <map>

namespace pkg
{

struct Implementation
{
	std::string name;
	std::string version;

	Implementation(const std::string& name, const std::string& version)
		: name(name), version(version) {}
};

/**
 * Package storage implementation to support the Package flyweight objects.
 *
 * This interface is only used by package.cc and should be never exported; this
 * allows to change the Store backend to anything with different performance
 * requirements, at any time.
 */
struct Store
{
	typedef std::map<std::string, Implementation*> versionmap_t;
	typedef std::map<std::string, versionmap_t> store_t;
	store_t store;

	/**
	 * Get a versionless package
	 */
	Implementation* get(const std::string& name)
	{
		return get(name, std::string());
	}

	/**
	 * Get a package with a version
	 */
	Implementation* get(const std::string& name, const std::string& version)
	{
		store_t::iterator pkg = store.find(name);
		if (pkg == store.end())
		{
			std::pair<store_t::iterator, bool> tmp = store.insert(std::make_pair(name, versionmap_t()));
			pkg = tmp.first;
		}
		versionmap_t::iterator ver = pkg->second.find(version);
		if (ver == pkg->second.end())
		{
			std::pair<versionmap_t::iterator, bool> tmp = pkg->second.insert(std::make_pair(version, new Implementation(name, version)));
			ver = tmp.first;
		}
		return ver->second;
	}

	/**
	 * Report that one package flyweight object went out of scope
	 */
	void release(Implementation* item) {}
};

};

// vim:set ts=4 sw=4:
#endif
